use std::rc::Rc;
use std::result::Result as StdResult;

use wasm_bindgen::prelude::*;
use wasm_bindgen::JsCast;

pub mod engine;

/// A type that makes dealing with results easier.
pub type Result<T> = StdResult<T, JsValue>;

#[wasm_bindgen]
extern "C" {
    #[wasm_bindgen(js_namespace = console, js_name = log)]
    pub fn console_log(s: &str);

    #[wasm_bindgen(js_namespace = console, js_name = log)]
    pub fn console_log_js(s: JsValue);
}

macro_rules! log {
    ($($t:tt)*) => (crate::console_log(&format_args!($($t)*).to_string()))
}

macro_rules! log_js {
    ($t:tt) => {{
        let e: JsValue = $t.into();
        crate::console_log_js(e)
    }};
}

pub(crate) use log;
pub(crate) use log_js;

#[wasm_bindgen(start)]
pub fn start() -> Result<()> {
    let mut engine = engine::Engine::new()?;
    engine.start()?;
    Ok(())
}
